<?php
// www.p30vel.ir is the Best Publisher 

if(!defined("access")){die();}
Class Register{
	
	//account username
	private $username;
	//account firstname
	private $firstname;
	//account lastname
	private $lastname;
	//account password1
	private $password1;
	//account password2
	private $password2;
	//account email
	private $email;
	//and accounts gender
	private $gender;
	private $activate_os;
	
	//errors we will be showing to the user
	private $error = '';
	//which field have errors so they get a red border
	private $error_fields = '';
	
	/* Initializes the variables that we will be needing for the registration process
	 * $data Array - contains all user info of the form
	*/
	function __construct($data){
		$misc = new Misc();
		foreach ($data as $id => $val){
			//assign each item to its variable
			$this->$id = $misc->trimValue($val);//remove all spaces
		}
	}
	
	/* Validates the user data
	 * all variables are initialized from the constructor above
	 * returns errors if found and registers user if not
	*/
	public function registerUser(){
		$misc = new Misc(); //get the Misc Class
		
		//are the variables blank? throw error
		if ($this->username != "" && $this->firstname != "" && $this->lastname != "" &&
			$this->password1 != "" && $this->password2 != "" && $this->email != "" &&
			$this->gender != "")
		{
			
			//check if username chars length < 1 or > 20 and does not meet our pattern, throw an error
			if (strlen($this->username) < 1 || strlen($this->username) > 20){
				$this->error .= '<li>Your Username must be between 1 and 20 characters</li>';
				$this->error_fields .= 'username,';
			}
			
			//pattern for username
			$pattern = "/[^0-9A-Za-z]/";
			$validator = preg_match($pattern, $this->username);
			if ($validator == 1){
				$this->error .= '<li>Your Username characters must be a-z A-Z 0-9</li>';
				$this->error_fields .= 'username,';
			}
			
			//die($validator);
			//does the username exist?if yes then throw an error since we dont allow duplicate usernames
			if ($misc->usernameExist(mysql_real_escape_string($this->username))){
				$this->error .= '<li>This username is already taken.</li>';
				$this->error_fields .= 'username,';
			}
			
			//1 to 100 characters count for firstname
			if (strlen($this->firstname) < 1 || strlen($this->firstname) > 100){
				$this->error .= '<li>Your Firstname must be between 1 and 100 characters.</li>';
				$this->error_fields .= 'firstname,';
			}
			
			//1 to 100 characters count for lastname
			if (strlen($this->lastname) < 1 || strlen($this->lastname) > 100){
				$this->error .= '<li>Your Lastname must be between 1 and 100 characters.</li>';
				$this->error_fields .= 'lastname,';
			}
			
			//if password1 does not meet our criteria for the password (at least 6 characters length with maximum
			//of 128 characters and alphanumeric throw an error
			if (!ctype_alnum($this->password1)
				|| strlen($this->password1)<6
				|| strlen($this->password1)>=128
				|| !preg_match('`[A-Z]`',$this->password1)
				|| !preg_match('`[a-z]`',$this->password1)
				|| !preg_match('`[0-9]`',$this->password1)
			){
				$this->error .= '<li>Your first password must have at least one of each: a-z A-Z 0-9 and be atleast 6 characters.</li>';
				$this->error_fields .= 'password,';
			}
			
			//if password2 does not meet our criteria for the password (at least 6 characters length with maximum
			//of 128 characters and alphanumeric throw an error
			if (!ctype_alnum($this->password2)
				|| strlen($this->password2)<6
				|| strlen($this->password2)>=128
				|| !preg_match('`[A-Z]`',$this->password2)
				|| !preg_match('`[a-z]`',$this->password2)
				|| !preg_match('`[0-9]`',$this->password2)
			){
				$this->error .= '<li>Your second password must have at least one of each: a-z A-Z 0-9 and be atleast 6 characters.</li>';
				$this->error_fields .= 'retype_password,';
			}
			
			//is the email address valid? if not throw an error
			if(preg_match( "/^[\d\w\/+!=#|$?%{^&}*`'~-][\d\w\/\.+!=#|$?%{^&}*`'~-]*@[A-Z0-9][A-Z0-9.-]{1,61}[A-Z0-9]\.[A-Z]{2,6}$/ix", $this->email ) == 0){
				$this->error .= '<li>Please enter a valid Email address.</li>';
				$this->error_fields .= 'email,';
			}
			
			//if the email already exist , throw an error... we dont need duplicates as well
			if ($misc->emailExist(mysql_real_escape_string($this->email))){
				$this->error .= '<li>This email is already taken.</li>';
				$this->error_fields .= 'email,';
			}
			
			//if gender is not a Male or Female, tell the user to Select one.
			if ($this->gender != 1 && $this->gender != 2){
				$this->error .= '<li>Please Select your Gender.</li>';
				$this->error_fields .= 'gender,';
			}
			
			//if no errors found , we are good... for now ;)
			if ($this->error == ""){
				//we gotta check if the passwords match
				if ($this->password1 == $this->password2){
					//and if the user has entered a valid gender... user can easily change the values from the HTML DOM
					if ($this->gender == 1 || $this->gender == 2){
						//make an array with all the data for the account
						$array = array(
							'username' => $misc->escapeValue($this->username), //escape variables
							'firstname' => $misc->escapeValue($this->firstname),
							'lastname' => $misc->escapeValue($this->lastname),
							'password1' => $misc->escapeValue($this->password1),
							'password2' => $misc->escapeValue($this->password2),
							'email' => $misc->escapeValue($this->email),
							'gender' => $misc->escapeValue($this->gender)
						);
						//then register the user
						$this->addUser($array);
						
						if ($this->activate_os == "checked"){
							if (isset($_SESSION["logged_as_admin"])){
								mysql_query("UPDATE users SET user_activated='1' WHERE user_name='$this->username' && user_activated='0'");
							}
						}else{
							if ($misc->approve_accounts == false){
								$activate = new Activate($this->email, null);
								//insert the activation code into the database and sent to the user an email with the code in it in order
								//to activate the acount.
								$hash = $activate->insertActivation();
							}
						}
					}else{
						$this->error .= '<li>Please Select your Gender.</li>';
						$this->error_fields .= 'gender,';
					}
				}else{
					$this->error .= '<li>Your passwords do not match.</li>';
					$this->error_fields .= 'password,retype_password';
				}
			}
		}else{
			//all fields are blank, tell the user to fill them.
			$this->error .= '<li>Please fill in all fields.</li>';
			//red marked fields
			$this->error_fields .= 'username,firstname,lastname,password,retype_password,email,gender';
		}
		
		//no errors for sure(hopefully)
		if ($this->error == ""){
			//send user a message that the account is registered
			$array = array(
				'error' => false,
				'message' => 'Successfully registered',
				'inputs_errors' => ''
			);
		}else{
			//else send the errors
			$array = array(
				'error' => true,
				'message' => '<ul style="padding:0;margin:0;margin-left:15px;">'.$this->error.'</ul>',
				'inputs_errors' => $this->error_fields
			);
		}
		return $array;
	}
	
	/* @param $data Array - Contains users data
	 * Registers the users account
	 * returns nothing
	*/
	private function addUser($data){
		$misc = new Misc();
		//generate the users id
		$user_id = $misc->generateId();
		$flag = false;
		//until flag is false
		while ($flag == false){
			//check if the user id exists
			$q = mysql_query("SELECT * FROM users WHERE user_id='$user_id'") or die(mysql_error());
			$rows = mysql_num_rows($q);
			//if it does try again till you find an id that does not exist
			if ($rows){
				$user_id = $misc->generateId();
			}else{
				//if it does not exist, exit the loop
				$flag = true;
			}
		}
		if ($flag == true){
			//md5 password
			$password = md5($this->password1);
			if ($this->gender == 1){
				$avatar = "photos/default_male.png";
			}else{
				$avatar = "photos/default_female.png";
			}
			//insert into db the data
			mysql_query("INSERT INTO users VALUES ('', '$user_id', '$this->firstname', '$this->lastname', '$this->gender', '$this->username', '$password', '$this->email', '$avatar', '0')");
		}
	}

}
?>